% Copyright October, 2005, Brown University, Providence, RI. 
% All Rights Reserved 

% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a commercial
% product is hereby granted without fee, provided that the above copyright
% notice appear in all copies and that both that copyright notice and this
% permission notice appear in supporting documentation, and that the name of
% Brown University not be used in advertising or publicity pertaining to
% distribution of the software without specific, written prior permission. 

% BROWN UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY BE LIABLE FOR ANY
% SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
% CONNECTION WITH THE USE.

% Author: Frank Wood fwood@cs.brown.edu.

% function [trimmed_label, rate_history_data_structure, trimmed_rate,
% trimmed_kin] = build_history_data_structure(label, rate, kin, history_bins_to_use)
%
% takes label, rate, kin matrices and produces a history datastructure and
% appropriately trimmed label, rate, and kin matrices
%
%
% for example if history_bins_to_use = 10; 
% trimmed_label = label(10:end);
% rate_history_data_structure(1) = rate(1:10);
% rate_history_data_structure(2) = rate(2:11);
% rate_history_data_structure(3) = rate(3:12);
% ...
% trimmed_rate = rate(10:end);
% trimmed_kin  = kin(10:end);

function [trimmed_label, rate_history_data_structure, trimmed_rate, trimmed_kin] = build_history_data_structure(label, rate, kin, history_bins_to_use)

dataset_size = size(label, 1);

if (history_bins_to_use<1) 
    ERROR('::build_history_data_structure:: called with history_bins_to_use<1');
elseif (history_bins_to_use>=dataset_size)
    ERROR('::build_history_data_structure:: called with history_bins_to_use > dataset_size');
end

trimmed_label = label(history_bins_to_use:end,:);
trimmed_kin = kin(history_bins_to_use:end,:);
trimmed_rate = rate(history_bins_to_use:end,:);
rate_history_data_structure = zeros(dataset_size - history_bins_to_use+1,size(rate,2)*history_bins_to_use);

for i = history_bins_to_use:dataset_size     
    % we will concatenate all firing rates together column-wise

    rate_history_data_structure(i-history_bins_to_use+1,:) = reshape(rate(i:-1:i-history_bins_to_use+1,:),size(rate_history_data_structure(i-history_bins_to_use+1,:)));
    if (rem(i,1000)==0)
        disp(sprintf('::build_history_data_structure:: done with %d of %d',i,dataset_size));
    end
end

trimmed_label = trimmed_label';