% Copyright October, 2005, Brown University, Providence, RI. 
% All Rights Reserved 

% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a commercial
% product is hereby granted without fee, provided that the above copyright
% notice appear in all copies and that both that copyright notice and this
% permission notice appear in supporting documentation, and that the name of
% Brown University not be used in advertising or publicity pertaining to
% distribution of the software without specific, written prior permission. 

% BROWN UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY BE LIABLE FOR ANY
% SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
% CONNECTION WITH THE USE.

% Author: Frank Wood fwood@cs.brown.edu.

% function [corr_x, corr_y, mse_x, mse_y, mse_xy] =
% compute_decoding_accuracy(true, estimate, switching_state, plot_data, use_switching, Perceptron_state)


% utility function to take
% the Original trajectory and the estimatetructed trajectory

% - plot the graphs
% - output /return the MSE
% - output/return the corrcoeff

function [corr_x, corr_y, mse_x, mse_y, mse_xy] = compute_and_plot_decoding_accuracy(true, estimate, switching_state, plot_data, Perceptron_state)

start_time = 1;
end_time = size(true, 1);

true_X = true(:,1);
true_Y = true(:,2);
estimate_X = estimate(:,1);
estimate_Y = estimate(:,2);

% now that we have both true and estimatetructed data, check the quality

cc_x = corrcoef(true_X(start_time:end_time), estimate_X(start_time:end_time));
corr_x = cc_x(1,2);
disp(['Test data :: The c.c. btw. actual and estimatet. X traj. was ' num2str(corr_x)])
cc_y = corrcoef(true_Y(start_time:end_time), estimate_Y(start_time:end_time));
corr_y = cc_y(1,2);
disp(['Test data :: The c.c. btw. actual and estimatet. Y traj. was ' num2str(corr_y)])

% [x_sp1, x_sp2] =  my_convoluted_correlation(true_X, estimate_X, start_time, end_time);
% [y_sp1, y_sp2] =  my_convoluted_correlation(true_Y, estimate_Y, start_time, end_time);
% 
% disp(sprintf('Test Data:: Weighted correlation coefficients are %f %f', x_sp1, y_sp1));
% disp(sprintf('Test Data:: Cumsum correlation coefficients are %f %f', x_sp2, y_sp2));

T = size(estimate_X, 1);
eX = estimate_X(start_time:end_time) - true_X(start_time:end_time);
eY = estimate_Y(start_time:end_time) - true_Y(start_time:end_time);

mse_x = (1/T)*sum(eX.*eX);
mse_y = (1/T) * sum(eY.*eY);

mse_xy = mse_x+mse_y;
disp(sprintf('Test data:: Mean Squared Error is (x,y,xy) (%f,%f,%f)', mse_x, mse_y, mse_xy));

evX = diff(estimate_X(start_time:end_time)) - diff(true_X(start_time:end_time));
evY = diff(estimate_Y(start_time:end_time)) - diff(true_Y(start_time:end_time));

T = size(evX,1);
mse_vx = (1/T) * sum(evX .* evX);
mse_vy = (1/T) * sum(evY .* evY);
mse_vxy = mse_vx + mse_vy;
disp(sprintf('Test Data:: VELOCITY MSE is %f %f %f ', mse_vx, mse_vy, mse_vxy));

% show plots if the flag is set.. this is to disable plotting if you are
% varying parameters..

if (plot_data)
    figure(1)

    h1=plot(start_time:end_time, estimate_X(start_time:end_time),'k:');

    hold on
    h2=plot(start_time:end_time, true_X(start_time:end_time), 'k-');
    h3=plot(start_time:end_time, 2*switching_state(start_time:end_time)-20,'k:');
    h4=plot_perceptron_state(Perceptron_state,start_time, end_time);
    set(gca,'YTick',[])
    set(gca,'XTick',[])
    set(gca,'Xlim',[start_time end_time])
    hold off
    set(gca,'Xlim',[start_time end_time])
    
    figure(2)
    h1=plot(start_time:end_time, estimate_Y(start_time:end_time),'k:');
    hold on
    h2=plot(start_time:end_time, true_Y(start_time:end_time), 'k-');
    h3=plot(start_time:end_time, 2*switching_state(start_time:end_time)-20,'k:');
    h4=plot_perceptron_state(Perceptron_state,start_time, end_time);
    hold off

    set(gca,'YTick',[])
    set(gca,'XTick',[])
    set(gca,'Xlim',[start_time end_time])
    
end



function h4 = plot_perceptron_state(Perceptron_state,start_time, end_time)

cpi=1;
ps_points(cpi,1) = start_time;
ps_points(cpi,2) = 2*Perceptron_state(start_time)-15;
cpi=2;
for(i=start_time+1:end_time)
    if(Perceptron_state(i)~=Perceptron_state(i-1))
        ps_points(cpi,1) = i-.5;
        ps_points(cpi,2) = ps_points(cpi-1,2);
        cpi=cpi+1;
        ps_points(cpi,1) = i-.5;
        ps_points(cpi,2) = 2*Perceptron_state(i)-15;
        cpi=cpi+1;
    end
end

ps_points(cpi,1) = end_time;
ps_points(cpi,2) = ps_points(cpi-1,2);
h4=line(ps_points(:,1),ps_points(:,2));
set(h4,'Color',[0 0 0])