% Copyright October, 2005, Brown University, Providence, RI. 
% All Rights Reserved 

% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a commercial
% product is hereby granted without fee, provided that the above copyright
% notice appear in all copies and that both that copyright notice and this
% permission notice appear in supporting documentation, and that the name of
% Brown University not be used in advertising or publicity pertaining to
% distribution of the software without specific, written prior permission. 

% BROWN UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY BE LIABLE FOR ANY
% SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
% CONNECTION WITH THE USE.

% Author: Frank Wood fwood@cs.brown.edu.

% function [H,Q, A,W] = linear_gaussian_least_squares_fit(training_data)
% 
% and then fit the H,Q and A,W matrices
% z' ~ N(Hx,Q)
% x' ~ N(Ax,W)
% to the data. 

function [H,Q, A,W] = linear_gaussian_least_squares_fit(training_data)

attending_indexes = find(training_data.true_label==-1);
rate = training_data.rate(attending_indexes,:);
kin = training_data.kin(attending_indexes,:);

X = kin';
X_1 = X(:,1:end-1);
X_2 = X(:,2:end);
Z = rate';

A = X_2*X_1'*inv(X_1*X_1');
H = Z*X'*inv(X*X');
M = size(training_data.kin,1);
W = (X_2 - A*X_1)*(X_2 - A*X_1)' / (M-1);
Q = (Z - H*X)*(Z - H*X)'/ M;
